import { world } from "@minecraft/server";

const SPINNET = {};
/**
 * make a debug
 * @param {any} t data need debug
 * @param {EntityQuerryOption} e
 * @return {void}
 */
world.debug= (t,e={},r=false) => {
    const tell = m => {
      for (let d of world.getPlayers(e)) d.sendMessage(m);
    },
    o = t?.constructor?.name;
    let line;
    try {
        let u = void 0;
        u.clear();
    } catch (e) {
        line = `${e.stack}`.match(/\d+/g)[1];
    }
    
    let data = o?.includes("Error") ? `§4[Debugger Error]§c ${t}\n${t.stack}` : `§4[Debugger<§eLine: §f${line}§c> - Class: ${o ?? "None"}]§r ${JSON.colorStringify(t)}`;
    return r ? data : tell(data);
}

world.getEntities = (option = {}) => world.getDimension("overworld").getEntities(option);

function prototypeParser(item) {
  try {
    return item?.constructor?.name ?? (typeof item === "object" ? Object.getPrototypeOf(item)?.constructor?.name : void 0);
  } catch {
    return 'error';
  }
}
/**
 * Convert json to jsonstring color highlight
 * @private
 * @return {Object<String|Object>}
 */
export function JsonString(d) {
    try {
        let b = Array.isArray(d),
            out = b ? []: {},
            keyC = b ? "" : "§4∆ ",
            pIs = "Pending",
            cache;
        for (let k in d) switch (prototypeParser(d[k])) {
            case 'error':
                out[keyC + k] = `§go [Unable to load]`
                break;
            case undefined:
            case null:
                out[keyC + k] = `§6o ${d[k]}`
                break;
            case "String":
                out[keyC + k] = `§a$ ${d[k]}`
                break;
            case "Number":
                out[keyC + k] = `§s ${d[k]}`
                break;
            case "Boolean":
                out[keyC + k] = `§${d[k] ? "a" : "c"}o ${d[k]}`
                break;
            case "Array":
            case "Object":
                out[keyC + k] = JsonString(d[k]);
                break;
            case "Map":
                out[keyC + k] = JsonString(Array.from(d[k].entries()));
                break;
            case "WeakMap":
                out[keyC + k] = "§4o [Cannot Read WeakMap]";
                break;
            case "Set":
                out[keyC + k] = JsonString(Array.from(d[k]));
                break;
            case "WeakSet":
                out[keyC + k] = "§4o [Cannot Read WeakSet]";
                break;
            case "Date":
                out[keyC + k] = `§e ${d[k].toString()}`;
                break;
            case "Promise":
                d[k].then(() => (pIs = "Resolve"), () => (pIs = "Reject"));
                out[keyC + k] = `§d [Promise ${pIs}]`;
                break;
            case "Function":
                cache = SPINNET[d.constructor?.name]?.[k];
                out[keyC + k] = /function \(\) \{\n    \[native code\]\n\}/gm.exec(d[k].toString()) ? (cache ? `§e (${cache.param}) => ${cache.return}` : "§u [Native Function]") : ("§t$ " + d[k].toString().replace(/\s{4,}/g, ""));
                break;
            case "GeneratorFunction":
                out[keyC + k] = "§d Iterator<Unknow>"
                break;
            default:
               out[keyC + (b ? "" : `§0${d[k].constructor.name}0§ `) + k] = JsonString(d[k]);
        }
        return out;
    } catch (e) {
        console.error(e, e.stack)
    }
}

/**
 * Add minecraft color for you object
 * @param {any} data data for format
 * @param {number} space space
 * @return {String}
 */
JSON.colorStringify = function(data, space = 4) {
  if (!(typeof data === "object")) return `§6${data}`;
  return JSON.stringify(JsonString(data), void 0, space).replace(/\"§(\w+?|\w+?(∆|\$))\s(§0[\s\S]+?0§(\s*)|)([\s\S]*?)\"(:\s|,|$)/gm, (_, color, type, className, isKey, name, kind) => (_ = (isKey = type === "∆") || type === "$", `§${color.replace(/[∆$]/g, "").split("").join("§")}${type ? '"' : ""}${name}${type ? '"' : ""}§r${isKey ? `: ${className?.length ? `§8[§e${className.slice(2, -3)}§8]§l >§r ` : ""}` : kind?.length ? "," : ""}`));
};

/**
 * Makes the first letter of a string uppercase.
 * @this {String}
 * @return {string} The string with the first letter in uppercase.
 */
String.prototype.firstUpperCase=function(){return this[0].toUpperCase()+this.slice(1)},

/**
 * Removes the namespace from a Minecraft resource location.
 * @this {String}
 * @return {string} The string without the namespace.
 */
String.prototype.removeNameSpace=function(){return this.split(":")?.[1] ?? this},

/**
 * Replaces underscores in a string with spaces.
 * @this {String}
 * @return {string} The string with underscores replaced with spaces.
 */
String.prototype.removeUnderScope=function(){return this.split("_").join(" ")},

/**
 * Converts a Minecraft resource location string to a more readable format with proper casing.
 * @this {String}
 * @return {string} The formatted string.
 */
String.prototype.idToString=function(){return (this.split(":")[1] || this).split("_").map(str => str[0].toUpperCase() + str.slice(1)).join(" ") },

/**
 * Converts a string to a pseudo-random hexadecimal identifier.
 * @this {String}
 * @return {string} The identifier.
 */
String.prototype.toId=function(){
    let t = e =>{
        let r=e.replace(/[xyz]/g,t=>{
            let e=(new Date().getTime()+16*Math.random())%16|0;
            return("x"==t?e:"z"==t?(e>8?"-":"_"):3&e|8).toString(16)
        });
        return Number.isNaN(+r)?r:t(e);
    };
    return t(this);
}